function [distrib, valores] = distrib_suma_dos_va(xpart,xval,ypart,yval)
%  [distrib,valores] = distrib_suma_dos_va(xpart,xval,ypart,yval)
%  Dadas dos var aleat (X,Y) en forma particional, hallar distrib de X+Y
%  Las variables estan definidas sobre el espacio muestral [0,1],
%     con una distribucion uniforme
%  xpart: vector (fila o columna) de puntos que definen la particion de X
%         1er elemento: 0, ultimo elemento: 1, orden ascendiente
%  ypart: vector (fila o columna) de puntos que definen la particion de Y
%         1er elemento: 0, ultimo elemento: 1, orden ascendiente
%  xval:  valores para los distintos elementos de la particion de X
%  yval:  valores para los distintos elementos de la particion de Y
%  valores: valores _distintos_ que toma la suma X + Y
%  distrib: distribucion de la suma X + Y
% 
%  Ejemplo:  xpart = [0 1/4 1/3 1/2 3/4 1]  xval =  [10  15  20  15  10]
%            ypart = [0 1/3 5/6 1]  yval = [3 2 3]
%  El output es: distrib = [1/12 5/12 3/12 1/12 2/12]
%                valores = [ 12   13   17   18   22 ]

%% Funcion de Matlab/Octave -- Ricard Torres, 2015 (sin acentos)

% En primer lugar, comprobamos que los argumentos sean correctos
if ( nargin ~= 4 )
   mostrar_uso() ;
   error('La funcion debe tener cuatro argumentos') ;
end

if ~ and( isvector(xpart), and(isvector(xval), and(isvector(ypart), ...
     isvector(yval) ) ) )
   mostrar_uso() ;
   error('Los argumentos de la funcion deben de ser vectores') ;
end

lpx = length(xpart); lvx = length(xval);
lpy = length(ypart); lvy = length(yval);

if ~ and( ( lvx == lpx - 1 ),  ( lvy == lpy - 1 ) ) 
   mostrar_uso() ;
   error('Debe de haber un valor para cada elemento de la particion') ;
end

if ~ and( (xpart(1) == 0), and( (xpart(end) == 1), and( (ypart(1) == 0), ...
    (ypart(end) == 1) ) ) ) 
   mostrar_uso() ;
   error('Cada particion debe empezar con 0 y acabar con 1') ;
end

is_ascending = true;
for i = 2 : lpx
    if xpart(i) <= xpart(i-1), is_ascending = false; end
end
for i = 2 : lpy
    if ypart(i) <= ypart(i-1), is_ascending = false; end
end

if ~ is_ascending
   mostrar_uso() ;
   error('El vector que define cada particion debe tener orden ascendiente') ;
end

% Ahora desarrollamos los calculos propiamente dichos
% Asegurar que ambos vectores de particion son vectores fila
xpart = reshape(xpart,1,lpx);
ypart = reshape(ypart,1,lpy);
% Hallar la menor particion que refina tanto a xpart como a ypart
partcomun = unique(sort([xpart,ypart]));
% Hallar los valores de la suma para cada elemento de la particion comun
lpc = length(partcomun);
for i = 2 : lpc
    j = 2;
    while ( xpart(j) < partcomun(i) )
          j = j + 1;
    end
    k = 2;
    while ( ypart(k) < partcomun(i) )
          k = k + 1;
    end
    valxmasy(i-1) = xval(j-1) + yval(k-1);
end
% Agrupar las repeticiones
valores = unique(sort(valxmasy));
% Hallar la distribucion
lv = length(valores);
distrib = zeros(1,lv);
for i = 1 : lv
    for j = 1 : lpc - 1
        if ( valores(i) == valxmasy(j) )
           distrib(i) = distrib(i) + partcomun(j+1) - partcomun(j);
        end
    end
end

end

% Mensaje de ayuda en caso de error
function mostrar_uso()
fprintf('Uso: [distrib,valores] = distrib_suma_dos_va(xpart,xval,ypart,yval)\n');
fprintf('Dadas dos var aleat (X,Y) en forma particional, ');
fprintf('hallar distrib de X+Y\n');
fprintf('Las variables estan definidas sobre el espacio muestral [0,1],\n');
fprintf('  con una distribucion uniforme\n');
fprintf('xpart: vector (fila o columna) de puntos que definen la particion de X\n');
fprintf('    1er elemento: 0, ultimo elemento: 1, orden ascendiente\n');
fprintf('ypart: vector (fila o columna) de puntos que definen la particion de Y\n');
fprintf('    1er elemento: 0, ultimo elemento: 1, orden ascendiente\n');
fprintf('xval:  valores para los distintos elementos ');
fprintf(' de la particion de X\n');
fprintf('yval:  valores para los distintos elementos ');
fprintf(' de la particion de Y\n');
fprintf('valores: valores _distintos_ que toma la suma X + Y\n');
fprintf('distrib: distribucion de la suma X + Y\n');
fprintf(' ie, probabilidad de cada par de valores de los vectores valx y valy\n');
fprintf('\nEjemplo:  xpart = [0 1/4 1/3 1/2 3/4 1] xval =  [10  15  20  15  10]\n');
fprintf('            ypart = [0 1/3 5/6 1]  yval = [3 2 3]\n');
fprintf('El output es: distrib = [1/12 5/12 3/12 1/12 2/12]\n');
fprintf('              valores = [ 12   13   17   18   22 ]\n');
end

